/*
 * Copyright [2013-2021], Alibaba Group Holding Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.polardbx.server.conn;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;

/**
 * 参见InnerConnection
 *
 * @author ziyang.lb 2020-12-05
 */
public class InnerStatement implements Statement {

    protected final InnerConnection connection;

    public InnerStatement(InnerConnection innerConnection) {
        this.connection = innerConnection;
    }

    @Override
    public ResultSet executeQuery(String sql) throws SQLException {
        return (ResultSet) connection.executeSql(sql, null);
    }

    @Override
    public int executeUpdate(String sql) throws SQLException {
        return (int) connection.executeSql(sql, null);
    }

    @Override
    public void close() throws SQLException {
    }

    // ----------------------------------------------------------------------------------------------
    // ---------------------------------------根据需求，逐步实现----------------------------------------
    // ----------------------------------------------------------------------------------------------
    @Override
    public long getLargeUpdateCount() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void setLargeMaxRows(long max) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public long getLargeMaxRows() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public long[] executeLargeBatch() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public long executeLargeUpdate(String sql) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public long executeLargeUpdate(String sql, int autoGeneratedKeys) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public long executeLargeUpdate(String sql, int[] columnIndexes) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public long executeLargeUpdate(String sql, String[] columnNames) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int getMaxFieldSize() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void setMaxFieldSize(int max) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int getMaxRows() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void setMaxRows(int max) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void setEscapeProcessing(boolean enable) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int getQueryTimeout() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void setQueryTimeout(int seconds) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void cancel() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public SQLWarning getWarnings() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void clearWarnings() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void setCursorName(String name) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public boolean execute(String sql) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public ResultSet getResultSet() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int getUpdateCount() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public boolean getMoreResults() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void setFetchDirection(int direction) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int getFetchDirection() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void setFetchSize(int rows) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int getFetchSize() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int getResultSetConcurrency() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int getResultSetType() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void addBatch(String sql) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void clearBatch() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int[] executeBatch() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public Connection getConnection() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public boolean getMoreResults(int current) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public ResultSet getGeneratedKeys() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int executeUpdate(String sql, int autoGeneratedKeys) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int executeUpdate(String sql, int[] columnIndexes) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int executeUpdate(String sql, String[] columnNames) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public boolean execute(String sql, int autoGeneratedKeys) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public boolean execute(String sql, int[] columnIndexes) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public boolean execute(String sql, String[] columnNames) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public int getResultSetHoldability() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public boolean isClosed() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void setPoolable(boolean poolable) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public boolean isPoolable() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public void closeOnCompletion() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public boolean isCloseOnCompletion() throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public <T> T unwrap(Class<T> iface) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }

    @Override
    public boolean isWrapperFor(Class<?> iface) throws SQLException {
        throw new UnsupportedOperationException("unsupported operation");
    }
}
